@@ -1,16 +1,26 @@
+# This code is part of the Biopython distribution and governed by its
+# license.  Please see the LICENSE file that should have been included
+# as part of this package.
+#
+
 """Hold GenBank data in a straightforward format.
 
 classes:
-o Record - All of the information in a GenBank record.
-o Reference - hold reference data for a record.
-o Feature - Hold the information in a Feature Table.
-o Qualifier - Qualifiers on a Feature.
+
+    - Record - All of the information in a GenBank record.
+    - Reference - hold reference data for a record.
+    - Feature - Hold the information in a Feature Table.
+    - Qualifier - Qualifiers on a Feature.
+
 17-MAR-2009: added support for WGS and WGS_SCAFLD lines.  Ying Huang & Iddo Friedberg
 """
 # local stuff
 import Bio.GenBank
 
-def _wrapped_genbank(information, indent, wrap_space = 1, split_char = " "):
+__docformat__ = "restructuredtext en"
+
+
+def _wrapped_genbank(information, indent, wrap_space=1, split_char=" "):
     """Write a line of GenBank info that can wrap over multiple lines.
 
     This takes a line of information which can potentially wrap over
@@ -19,19 +29,23 @@ def _wrapped_genbank(information, indent, wrap_space = 1, split_char = " "):
 
     Arguments:
 
-    o information - The string holding the information we want
-    wrapped in GenBank method.
+        - information - The string holding the information we want
+          wrapped in GenBank method.
 
-    o indent - The indentation on the lines we are writing.
+        - indent - The indentation on the lines we are writing.
 
-    o wrap_space - Whether or not to wrap only on spaces in the
-    information.
+        - wrap_space - Whether or not to wrap only on spaces in the
+          information.
 
-    o split_char - A specific character to split the lines on. By default
-    spaces are used.
+        - split_char - A specific character to split the lines on. By default
+          spaces are used.
     """
     info_length = Record.GB_LINE_LENGTH - indent
 
+    if not information:
+        # GenBank files use "." for missing data
+        return ".\n"
+
     if wrap_space:
         info_parts = information.split(split_char)
     else:
@@ -40,7 +54,7 @@ def _wrapped_genbank(information, indent, wrap_space = 1, split_char = " "):
         while cur_pos < len(information):
             info_parts.append(information[cur_pos: cur_pos + info_length])
             cur_pos += info_length
-            
+
     # first get the information string split up by line
     output_parts = []
     cur_part = ""
@@ -66,8 +80,9 @@ def _wrapped_genbank(information, indent, wrap_space = 1, split_char = " "):
     for output_part in output_parts[1:]:
         output_info += " " * indent + output_part + "\n"
 
-    return output_info            
-        
+    return output_info
+
+
 def _indent_genbank(information, indent):
     """Write out information with the specified indent.
 
@@ -86,46 +101,48 @@ def _indent_genbank(information, indent):
 
     return output_info
 
-class Record:
+
+class Record(object):
     """Hold GenBank information in a format similar to the original record.
 
     The Record class is meant to make data easy to get to when you are
     just interested in looking at GenBank data.
 
     Attributes:
-    o locus - The name specified after the LOCUS keyword in the GenBank
-    record. This may be the accession number, or a clone id or something else.
-    o size - The size of the record.
-    o residue_type - The type of residues making up the sequence in this
-    record. Normally something like RNA, DNA or PROTEIN, but may be as
-    esoteric as 'ss-RNA circular'.
-    o data_file_division - The division this record is stored under in
-    GenBank (ie. PLN -> plants; PRI -> humans, primates; BCT -> bacteria...)
-    o date - The date of submission of the record, in a form like '28-JUL-1998'
-    o accession - list of all accession numbers for the sequence.
-    o nid - Nucleotide identifier number.
-    o pid - Proteint identifier number
-    o version - The accession number + version (ie. AB01234.2)
-    o db_source - Information about the database the record came from
-    o gi - The NCBI gi identifier for the record.
-    o keywords - A list of keywords related to the record.
-    o segment - If the record is one of a series, this is info about which
-    segment this record is (something like '1 of 6').
-    o source - The source of material where the sequence came from.
-    o organism - The genus and species of the organism (ie. 'Homo sapiens')
-    o taxonomy - A listing of the taxonomic classification of the organism,
-    starting general and getting more specific.
-    o references - A list of Reference objects.
-    o comment - Text with any kind of comment about the record.
-    o features - A listing of Features making up the feature table.
-    o base_counts - A string with the counts of bases for the sequence.
-    o origin - A string specifying info about the origin of the sequence.
-    o sequence - A string with the sequence itself.
-    o contig - A string of location information for a CONTIG in a RefSeq file
-    o project - The genome sequencing project numbers
-                (will be replaced by the dblink cross-references in 2009).
-    o dblinks - The genome sequencing project number(s) and other links.
-                (will replace the project information in 2009).
+
+        - locus - The name specified after the LOCUS keyword in the GenBank
+          record. This may be the accession number, or a clone id or something else.
+        - size - The size of the record.
+        - residue_type - The type of residues making up the sequence in this
+          record. Normally something like RNA, DNA or PROTEIN, but may be as
+          esoteric as 'ss-RNA circular'.
+        - data_file_division - The division this record is stored under in
+          GenBank (ie. PLN -> plants; PRI -> humans, primates; BCT -> bacteria...)
+        - date - The date of submission of the record, in a form like '28-JUL-1998'
+        - accession - list of all accession numbers for the sequence.
+        - nid - Nucleotide identifier number.
+        - pid - Proteint identifier number
+        - version - The accession number + version (ie. AB01234.2)
+        - db_source - Information about the database the record came from
+        - gi - The NCBI gi identifier for the record.
+        - keywords - A list of keywords related to the record.
+        - segment - If the record is one of a series, this is info about which
+          segment this record is (something like '1 of 6').
+        - source - The source of material where the sequence came from.
+        - organism - The genus and species of the organism (ie. 'Homo sapiens')
+        - taxonomy - A listing of the taxonomic classification of the organism,
+          starting general and getting more specific.
+        - references - A list of Reference objects.
+        - comment - Text with any kind of comment about the record.
+        - features - A listing of Features making up the feature table.
+        - base_counts - A string with the counts of bases for the sequence.
+        - origin - A string specifying info about the origin of the sequence.
+        - sequence - A string with the sequence itself.
+        - contig - A string of location information for a CONTIG in a RefSeq file
+        - project - The genome sequencing project numbers
+          (will be replaced by the dblink cross-references in 2009).
+        - dblinks - The genome sequencing project number(s) and other links.
+          (will replace the project information in 2009).
     """
     # constants for outputting GenBank information
     GB_LINE_LENGTH = 79
@@ -148,7 +165,7 @@ class Record:
                               str(GB_FEATURE_INDENT -
                                   GB_FEATURE_INTERNAL_INDENT) + "s"
     SEQUENCE_FORMAT = "%" + str(GB_SEQUENCE_INDENT) + "s"
-    
+
     def __init__(self):
         self.locus = ''
         self.size = ''
@@ -176,7 +193,7 @@ class Record:
         self.origin = ''
         self.sequence = ''
         self.contig = ''
-        self.primary=[]
+        self.primary = []
         self.wgs = ''
         self.wgs_scafld = []
 
@@ -220,36 +237,36 @@ class Record:
         output += self._contig_line()
         output += "//"
         return output
-            
+
     def _locus_line(self):
         """Provide the output string for the LOCUS line.
         """
         output = "LOCUS"
-        output += " " * 7 # 6-12 spaces
+        output += " " * 7  # 6-12 spaces
         output += "%-9s" % self.locus
-        output += " " # 22 space
+        output += " "  # 22 space
         output += "%7s" % self.size
-        if self.residue_type.find("PROTEIN") >= 0:
+        if "PROTEIN" in self.residue_type:
             output += " aa"
         else:
             output += " bp "
 
         # treat circular types differently, since they'll have long residue
         # types
-        if self.residue_type.find("circular") >= 0:
-             output += "%17s" % self.residue_type
+        if "circular" in self.residue_type:
+            output += "%17s" % self.residue_type
         # second case: ss-DNA types of records
-        elif self.residue_type.find("-") >= 0:
+        elif "-" in self.residue_type:
             output += "%7s" % self.residue_type
-            output += " " * 10 # spaces for circular
+            output += " " * 10  # spaces for circular
         else:
-            output += " " * 3 # spaces for stuff like ss-
+            output += " " * 3  # spaces for stuff like ss-
             output += "%-4s" % self.residue_type
-            output += " " * 10 # spaces for circular
+            output += " " * 10  # spaces for circular
 
         output += " " * 2
         output += "%3s" % self.data_file_division
-        output += " " * 7 # spaces for 56-63
+        output += " " * 7  # spaces for 56-63
         output += "%11s" % self.date
         output += "\n"
         return output
@@ -275,7 +292,7 @@ class Record:
             output += _wrapped_genbank(acc_info, Record.GB_BASE_INDENT)
         else:
             output = ""
-        
+
         return output
 
     def _version_line(self):
@@ -330,14 +347,14 @@ class Record:
         """
         output = ""
         if len(self.keywords) >= 0:
-            output +=  Record.BASE_FORMAT % "KEYWORDS"
+            output += Record.BASE_FORMAT % "KEYWORDS"
             keyword_info = ""
             for keyword in self.keywords:
                 keyword_info += "%s; " % keyword
             # replace the ; at the end with a period
             keyword_info = keyword_info[:-2]
             keyword_info += "."
-            
+
             output += _wrapped_genbank(keyword_info,
                                        Record.GB_BASE_INDENT)
 
@@ -368,7 +385,7 @@ class Record:
         output = Record.BASE_FORMAT % "SOURCE"
         output += _wrapped_genbank(self.source, Record.GB_BASE_INDENT)
         return output
-    
+
     def _organism_line(self):
         """Output for ORGANISM line with taxonomy info.
         """
@@ -385,7 +402,7 @@ class Record:
         output += _wrapped_genbank(taxonomy_info, Record.GB_BASE_INDENT)
 
         return output
-            
+
     def _comment_line(self):
         """Output for the COMMENT lines.
         """
@@ -463,7 +480,7 @@ class Record:
                     # stop looping if we are out of sequence
                     if end_pos > len(self.sequence):
                         break
-                
+
                 output += "\n"
                 cur_seq_pos += 60
         return output
@@ -481,7 +498,7 @@ class Record:
                     output += Record.BASE_FORMAT % "WGS_SCAFLD"
                     output += self.wgs_scafld
             return output
-        
+
     def _contig_line(self):
         """Output for CONTIG location information from RefSeq.
         """
@@ -489,23 +506,24 @@ class Record:
         if self.contig:
             output += Record.BASE_FORMAT % "CONTIG"
             output += _wrapped_genbank(self.contig,
-                                       Record.GB_BASE_INDENT, split_char = ',')
+                                       Record.GB_BASE_INDENT, split_char=',')
         return output
-        
 
-class Reference:
+
+class Reference(object):
     """Hold information from a GenBank reference.
 
     Attributes:
-    o number - The number of the reference in the listing of references.
-    o bases - The bases in the sequence the reference refers to.
-    o authors - String with all of the authors.
-    o consrtm - Consortium the authors belong to. 
-    o title - The title of the reference.
-    o journal - Information about the journal where the reference appeared.
-    o medline_id - The medline id for the reference.
-    o pubmed_id - The pubmed_id for the reference.
-    o remark - Free-form remarks about the reference.
+
+        - number - The number of the reference in the listing of references.
+        - bases - The bases in the sequence the reference refers to.
+        - authors - String with all of the authors.
+        - consrtm - Consortium the authors belong to.
+        - title - The title of the reference.
+        - journal - Information about the journal where the reference appeared.
+        - medline_id - The medline id for the reference.
+        - pubmed_id - The pubmed_id for the reference.
+        - remark - Free-form remarks about the reference.
     """
     def __init__(self):
         self.number = ''
@@ -527,7 +545,7 @@ class Reference:
         output += self._medline_line()
         output += self._pubmed_line()
         output += self._remark_line()
-        
+
         return output
 
     def _reference_line(self):
@@ -588,7 +606,7 @@ class Reference:
             output += Record.INTERNAL_FORMAT % "MEDLINE"
             output += self.medline_id + "\n"
         return output
-    
+
     def _pubmed_line(self):
         """Output for PUBMED information.
         """
@@ -597,7 +615,7 @@ class Reference:
             output += Record.OTHER_INTERNAL_FORMAT % "PUBMED"
             output += self.pubmed_id + "\n"
         return output
-    
+
     def _remark_line(self):
         """Output for REMARK information.
         """
@@ -606,14 +624,16 @@ class Reference:
             output += Record.INTERNAL_FORMAT % "REMARK"
             output += _wrapped_genbank(self.remark, Record.GB_BASE_INDENT)
         return output
-    
-class Feature:
+
+
+class Feature(object):
     """Hold information about a Feature in the Feature Table of GenBank record.
 
     Attributes:
-    o key - The key name of the featue (ie. source)
-    o location - The string specifying the location of the feature.
-    o qualfiers - A listing Qualifier objects in the feature.
+
+        - key - The key name of the featue (ie. source)
+        - location - The string specifying the location of the feature.
+        - qualfiers - A listing Qualifier objects in the feature.
     """
     def __init__(self):
         self.key = ''
@@ -623,27 +643,29 @@ class Feature:
     def __str__(self):
         output = Record.INTERNAL_FEATURE_FORMAT % self.key
         output += _wrapped_genbank(self.location, Record.GB_FEATURE_INDENT,
-                                   split_char = ',')
+                                   split_char=',')
         for qualifier in self.qualifiers:
             output += " " * Record.GB_FEATURE_INDENT
-            
+
             # determine whether we can wrap on spaces
             space_wrap = 1
             for no_space_key in \
-                Bio.GenBank._BaseGenBankConsumer.remove_space_keys:
-                if qualifier.key.find(no_space_key) >= 0:
+                    Bio.GenBank._BaseGenBankConsumer.remove_space_keys:
+                if no_space_key in qualifier.key:
                     space_wrap = 0
-            
+
             output += _wrapped_genbank(qualifier.key + qualifier.value,
                                        Record.GB_FEATURE_INDENT, space_wrap)
         return output
 
-class Qualifier:
+
+class Qualifier(object):
     """Hold information about a qualifier in a GenBank feature.
 
     Attributes:
-    o key - The key name of the qualifier (ie. /organism=)
-    o value - The value of the qualifier ("Dictyostelium discoideum").
+
+        - key - The key name of the qualifier (ie. /organism=)
+        - value - The value of the qualifier ("Dictyostelium discoideum").
     """
     def __init__(self):
         self.key = ''
