@@ -25,34 +25,30 @@
     objects - namely SeqRecord ojbects containing SeqFeature objects.
 """
 
-#------------------------------------------------------------------------------
+# -----------------------------------------------------------------------------
 # IMPORTS
 
 # ReportLab
-from reportlab.graphics import renderPS, renderPDF, renderSVG
 try:
     from reportlab.graphics import renderPM
 except ImportError:
-    #This is an optional part of ReportLab, so may not be installed.
-    renderPM=None
-from reportlab.lib import pagesizes
+    # This is an optional part of ReportLab, so may not be installed.
+    renderPM = None
 
 # GenomeDiagram
-from _LinearDrawer import LinearDrawer
-from _CircularDrawer import CircularDrawer
-from _Track import Track
-
-# Builtins
-import sys
+from ._LinearDrawer import LinearDrawer
+from ._CircularDrawer import CircularDrawer
+from ._Track import Track
 
 from Bio.Graphics import _write
 
-#------------------------------------------------------------------------------
+# -----------------------------------------------------------------------------
 # CLASSES
 
-#------------------------------------------------------------
+# -----------------------------------------------------------
 # Diagram
 
+
 class Diagram(object):
     """ Diagram
 
@@ -62,7 +58,7 @@ class Diagram(object):
 
         o name         String, identifier for the diagram
 
-        o tracks       List of Track objects comprising the diagram 
+        o tracks       List of Track objects comprising the diagram
 
         o format       String, format of the diagram (circular/linear)
 
@@ -70,24 +66,27 @@ class Diagram(object):
 
         o orientation  String, the page orientation (landscape/portrait)
 
-        o x            Float, the proportion of the page to take up with even 
+        o x            Float, the proportion of the page to take up with even
                               X margins
 
-        o y            Float, the proportion of the page to take up with even 
+        o y            Float, the proportion of the page to take up with even
                               Y margins
 
-        o xl           Float, the proportion of the page to take up with the 
+        o xl           Float, the proportion of the page to take up with the
                               left X margin
 
-        o xr           Float, the proportion of the page to take up with the 
+        o xr           Float, the proportion of the page to take up with the
                               right X margin
 
-        o yt           Float, the proportion of the page to take up with the 
+        o yt           Float, the proportion of the page to take up with the
                               top Y margin
 
-        o yb           Float, the proportion of the page to take up with the 
+        o yb           Float, the proportion of the page to take up with the
                               bottom Y margin
 
+        o circle_core  Float, the proportion of the available radius to leave
+                       empty at the center of a circular diagram (0 to 1).
+
         o start        Int, the base/aa position to start the diagram at
 
         o end          Int, the base/aa position to end the diagram at
@@ -97,24 +96,21 @@ class Diagram(object):
         o fragments    Int, for a linear diagram, the number of equal divisions
                                 into which the sequence is divided
 
-        o fragment_size Float, the proportion of the space available to each 
+        o fragment_size Float, the proportion of the space available to each
                                    fragment that should be used in drawing
 
-        o track_size   Float, the proportion of the space available to each 
+        o track_size   Float, the proportion of the space available to each
                                   track that should be used in drawing
 
-        o circular     Boolean, True if the genome/sequence to be drawn is, in 
-                                reality, circular.  
+        o circular     Boolean, True if the genome/sequence to be drawn is, in
+                                reality, circular.
 
         Methods:
 
         o __init__(self, name=None) Called on instantiation
 
-        o draw(self, format='circular', pagesize='A3', orientation='landscape',
-             x=0.05, y=0.05, xl=None, xr=None, yt=None, yb=None,
-             start=None, end=None, tracklines=0, fragments=10,
-             fragment_size=0.9, track_size=0.75) Instructs the package to draw
-             the diagram
+        o draw(self, format='circular', ...) Instructs the package to draw
+            the diagram
 
         o write(self, filename='test1.ps', output='PS') Writes the drawn
             diagram to a specified file, in a specified format.
@@ -148,11 +144,11 @@ class Diagram(object):
         o __str__(self) Returns a formatted string describing the diagram
 
     """
-    def __init__(self, name=None, format='circular', pagesize='A3', 
-         orientation='landscape', x=0.05, y=0.05, xl=None, 
-         xr=None, yt=None, yb=None, start=None, end=None, 
-         tracklines=False, fragments=10, fragment_size=0.9, 
-         track_size=0.75, circular=True):
+    def __init__(self, name=None, format='circular', pagesize='A3',
+         orientation='landscape', x=0.05, y=0.05, xl=None,
+         xr=None, yt=None, yb=None, start=None, end=None,
+         tracklines=False, fragments=10, fragment_size=0.9,
+         track_size=0.75, circular=True, circle_core=0.0):
         """ __init__(self, name=None)
 
             o name  String describing the diagram
@@ -192,13 +188,13 @@ class Diagram(object):
 
             o end       Int, the position to stop drawing the diagram at
 
-            o tracklines    Boolean flag to show (or not) lines delineating 
+            o tracklines    Boolean flag to show (or not) lines delineating
                         tracks on the diagram
 
             o fragments Int, for linear diagrams, the number of sections into
                         which to break the sequence being drawn
 
-            o fragment_size     Float (0->1), for linear diagrams, describing 
+            o fragment_size     Float (0->1), for linear diagrams, describing
                                 the proportion of space in a fragment to take
                                 up with tracks
 
@@ -207,7 +203,6 @@ class Diagram(object):
 
             o circular  Boolean flag to indicate whether the sequence being
                         drawn is circular
-                        
 
         """
         self.tracks = {}   # Holds all Track objects, keyed by level
@@ -229,6 +224,8 @@ class Diagram(object):
         self.fragment_size = fragment_size
         self.track_size = track_size
         self.circular = circular
+        self.circle_core = circle_core
+        self.cross_track_links = []
 
     def set_all_tracks(self, attr, value):
         """ set_all_tracks(self, attr, value)
@@ -243,48 +240,46 @@ class Diagram(object):
         for track in self.tracks.values():
             if hasattr(track, attr):          # If the feature has the attribute
                 if getattr(track, attr) != value:
-                    setattr(track, attr, value)   # set it to the passed value     
+                    setattr(track, attr, value)   # set it to the passed value
 
     def draw(self, format=None, pagesize=None, orientation=None,
              x=None, y=None, xl=None, xr=None, yt=None, yb=None,
              start=None, end=None, tracklines=None, fragments=None,
-             fragment_size=None, track_size=None, circular=None):
-        """ draw(self, format=None, pagesize=None, orientation=None,
-            x=None, y=None, xl=None, xr=None, yt=None, yb=None,
-            start=None, end=None, tracklines=None, fragments=None,
-            fragment_size=None, track_size=None)
-
-            Draws the diagram using the passed parameters, if any, to override
-        previous settings for the diagram object.
+             fragment_size=None, track_size=None, circular=None,
+             circle_core=None, cross_track_links=None):
+        """Draw the diagram, with passed parameters overriding existing attributes.
         """
-        # Pass the parameters to the drawer objects that will build the 
-        # diagrams.  At the moment, we detect overrides with an or in the 
-        # Instantiation arguments, but I suspect there's a neater way to do 
+        # Pass the parameters to the drawer objects that will build the
+        # diagrams.  At the moment, we detect overrides with an or in the
+        # Instantiation arguments, but I suspect there's a neater way to do
         # this.
         if format == 'linear':
-            drawer = LinearDrawer(self, pagesize or self.pagesize, 
-                                  orientation or self.orientation, 
-                                  x or self.x, y or self.y, xl or self.xl, 
-                                  xr or self.xr, yt or self.yt, 
-                                  yb or self.yb, start or self.start, 
-                                  end or self.end, 
+            drawer = LinearDrawer(self, pagesize or self.pagesize,
+                                  orientation or self.orientation,
+                                  x or self.x, y or self.y, xl or self.xl,
+                                  xr or self.xr, yt or self.yt,
+                                  yb or self.yb, start or self.start,
+                                  end or self.end,
                                   tracklines or self.tracklines,
-                                  fragments or self.fragments, 
-                                  fragment_size or self.fragment_size, 
-                                  track_size or self.track_size)
+                                  fragments or self.fragments,
+                                  fragment_size or self.fragment_size,
+                                  track_size or self.track_size,
+                                  cross_track_links or self.cross_track_links)
         else:
-            drawer = CircularDrawer(self, pagesize or self.pagesize, 
-                                    orientation or self.orientation, 
-                                    x or self.x, y or self.y, xl or self.xl, 
-                                    xr or self.xr, yt or self.yt, 
-                                    yb or self.yb, start or self.start, 
-                                    end or self.end, 
+            drawer = CircularDrawer(self, pagesize or self.pagesize,
+                                    orientation or self.orientation,
+                                    x or self.x, y or self.y, xl or self.xl,
+                                    xr or self.xr, yt or self.yt,
+                                    yb or self.yb, start or self.start,
+                                    end or self.end,
                                     tracklines or self.tracklines,
                                     track_size or self.track_size,
-                                    circular or self.circular)
+                                    circular or self.circular,
+                                    circle_core or self.circle_core,
+                                    cross_track_links or self.cross_track_links)
         drawer.draw()   # Tell the drawer to complete the drawing
         self.drawing = drawer.drawing  # Get the completed drawing
-        
+
     def write(self, filename='test1.ps', output='PS', dpi=72):
         """ write(self, filename='test1.ps', output='PS', dpi=72)
 
@@ -304,9 +299,9 @@ class Diagram(object):
             No return value.
         """
         return _write(self.drawing, filename, output, dpi=dpi)
-        
+
     def write_to_string(self, output='PS', dpi=72):
-        """ write(self, output='PS')
+        """Returns a byte string containing the diagram in the requested format.
 
             o output        String indicating output format, one of PS, PDF,
                             SVG, JPG, BMP, GIF, PNG, TIFF or TIFF (as
@@ -314,14 +309,17 @@ class Diagram(object):
 
             o dpi           Resolution (dots per inch) for bitmap formats.
 
-            Return the completed drawing as a string in a prescribed format
+            Return the completed drawing as a bytes string in a prescribed format
         """
-        #The ReportLab drawToString method, which this function used to call,
-        #just uses a cStringIO or StringIO handle with the drawToFile method.
-        #In order to put all our complicated file format specific code in one
-        #place we'll just use a StringIO handle here:
-        from StringIO import StringIO
-        handle = StringIO()
+        # The ReportLab drawToString method, which this function used to call,
+        # just used a cStringIO or StringIO handle with the drawToFile method.
+        # In order to put all our complicated file format specific code in one
+        # place we just used a StringIO handle here, later a BytesIO handle
+        # for Python 3 compatibility.
+        #
+        # TODO - Rename this method to include keyword bytes?
+        from io import BytesIO
+        handle = BytesIO()
         self.write(handle, output, dpi)
         return handle.getvalue()
 
@@ -335,18 +333,19 @@ class Diagram(object):
 
             Add a pre-existing Track to the diagram at a given level
         """
+        if track is None:
+            raise ValueError("Must specify track")
         if track_level not in self.tracks:     # No track at that level
             self.tracks[track_level] = track   # so just add it
         else:       # Already a track there, so shunt all higher tracks up one
-            occupied_levels = self.get_levels() # Get list of occupied levels...
-            occupied_levels.sort()              # ...sort it...
+            occupied_levels = sorted(self.get_levels())  # Get list of occupied levels...
             occupied_levels.reverse()           # ...reverse it (highest first)
-            for val in occupied_levels:     
-                if val >= track:        # If track value >= that to be added
-                    self.tracks[val+1] = self.tracks[val] # ...increment by 1
+            for val in occupied_levels:
+                # If track value >= that to be added
+                if val >= track.track_level:
+                    self.tracks[val + 1] = self.tracks[val]  # ...increment by 1
             self.tracks[track_level] = track   # And put the new track in
         self.tracks[track_level].track_level = track_level
-                
 
     def new_track(self, track_level, **args):
         """ new_track(self, track_level) -> Track
@@ -363,17 +362,15 @@ class Diagram(object):
         if track_level not in self.tracks:        # No track at that level
             self.tracks[track_level] = newtrack   # so just add it
         else:       # Already a track there, so shunt all higher tracks up one
-            occupied_levels = self.get_levels() # Get list of occupied levels...
-            occupied_levels.sort()              # ...sort it...
+            occupied_levels = sorted(self.get_levels())  # Get list of occupied levels...
             occupied_levels.reverse()           # ...reverse (highest first)...
-            for val in occupied_levels:     
+            for val in occupied_levels:
                 if val >= track_level:        # Track value >= that to be added
-                    self.tracks[val+1] = self.tracks[val] # ..increment by 1
+                    self.tracks[val + 1] = self.tracks[val]  # ..increment by 1
             self.tracks[track_level] = newtrack   # And put the new track in
         self.tracks[track_level].track_level = track_level
         return newtrack
 
-            
     def del_track(self, track_level):
         """ del_track(self, track_level)
 
@@ -383,14 +380,12 @@ class Diagram(object):
         """
         del self.tracks[track_level]
 
-
     def get_tracks(self):
         """ get_tracks(self) -> list
 
             Returns a list of the tracks contained in the diagram
         """
-        return self.tracks.values()
-
+        return list(self.tracks.values())
 
     def move_track(self, from_level, to_level):
         """ move_track(self, from_level, to_level)
@@ -406,7 +401,6 @@ class Diagram(object):
         del self.tracks[from_level]
         self.add_track(aux, to_level)
 
-
     def renumber_tracks(self, low=1, step=1):
         """ renumber_tracks(self, low=1, step=1)
 
@@ -418,11 +412,11 @@ class Diagram(object):
             value (low)
         """
         track = low                 # Start numbering from here
-        levels = self.get_levels()  # 
+        levels = self.get_levels()
 
         conversion = {}             # Holds new set of levels
         for level in levels:        # Starting at low...
-            conversion[track] = self.tracks[level] # Add old tracks to new set
+            conversion[track] = self.tracks[level]  # Add old tracks to new set
             conversion[track].track_level = track
             track += step                           # step interval
         self.tracks = conversion   # Replace old set of levels with new set
@@ -432,10 +426,7 @@ class Diagram(object):
 
             Return a sorted list of levels occupied by tracks in the diagram
         """
-        levels = self.tracks.keys()
-        levels.sort()
-        return levels
-
+        return sorted(self.tracks)
 
     def get_drawn_levels(self):
         """ get_drawn_levels(self) -> [int, int, ...]
@@ -443,11 +434,7 @@ class Diagram(object):
             Return a sorted list of levels occupied by tracks that are not
             explicitly hidden
         """
-        drawn_levels = [key for key in self.tracks.keys() if \
-                        not self.tracks[key].hide] # get list of shown levels
-        drawn_levels.sort()
-        return drawn_levels
-
+        return sorted(key for key in self.tracks if not self.tracks[key].hide)
 
     def range(self):
         """ range(self) -> (int, int)
@@ -456,7 +443,7 @@ class Diagram(object):
             track features as a tuple
         """
         lows, highs = [], []
-        for track in self.tracks.values(): # Get ranges for each track
+        for track in self.tracks.values():  # Get ranges for each track
             low, high = track.range()
             lows.append(low)
             highs.append(high)
@@ -481,5 +468,4 @@ class Diagram(object):
         for level in self.get_levels():
             outstr.append("Track %d: %s\n" % (level, self.tracks[level]))
         outstr = '\n'.join(outstr)
-        return outstr       
-
+        return outstr
