@@ -26,16 +26,44 @@ from Bio._py3k import urlopen as _urlopen
 from Bio.KEGG.KGML.KGML_pathway import Pathway
 
 
-def hexdarken(hexcolor, factor=0.7):
-    """Returns darkened hex color as a ReportLab RGB color.
+def darken(color, factor=0.7):
+    """Returns darkened color as a ReportLab RGB color.
 
-    Take a passed hex color and return an RGB color that is
-    slightly darker (if possible).
+    Take a passed color and returns a Reportlab color that is darker by the
+    factor indicated in the parameter.
     """
-    c = colors.HexColor(hexcolor)
+    newcol = color_to_reportlab(color)
     for a in ['red', 'green', 'blue']:
-        setattr(c, a, factor * getattr(c, a))
-    return c
+        setattr(newcol, a, factor * getattr(newcol, a))
+    return newcol
+
+
+def color_to_reportlab(color):
+    """Returns the passed color in Reportlab Color format.
+
+    We allow colors to be specified as hex values, tuples, or Reportlab Color
+    objects, and with or without an alpha channel. This function acts as a
+    Rosetta stone for conversion of those formats to a Reportlab Color
+    object, with alpha value.
+
+    Any other color specification is returned directly
+    """
+    # Reportlab Color objects are in the format we want already
+    if isinstance(color, colors.Color):
+        return color
+    elif isinstance(color, str):  # String implies hex color
+        if color.startswith("0x"):  # Standardise to octothorpe
+            color.replace("0x", "#")
+        if len(color) == 7:
+            return colors.HexColor(color)
+        else:
+            try:
+                return colors.HexColor(color, hasAlpha=True)
+            except TypeError:  # Catch pre-2.7 Reportlab
+                raise RuntimeError("Your reportlab seems to be too old, try 2.7 onwards")
+    elif isinstance(color, tuple):  # Tuple implies RGB(alpha) tuple
+        return colors.Color(*color)
+    return color
 
 
 def get_temp_imagefilename(url):
@@ -45,7 +73,7 @@ def get_temp_imagefilename(url):
     and return the filename.
     """
     img = _urlopen(url).read()
-    im = Image.open(BtyesIO(img))
+    im = Image.open(BytesIO(img))
     # im.transpose(Image.FLIP_TOP_BOTTOM)
     f = tempfile.NamedTemporaryFile(delete=False, suffix='.png')
     fname = f.name
@@ -62,12 +90,13 @@ class KGMLCanvas(object):
                  label_maps=True, show_maps=False, fontname='Helvetica',
                  fontsize=6, draw_relations=True, show_orthologs=True,
                  show_compounds=True, show_genes=True,
-                 margins=(0.02, 0.02)):
+                 show_reaction_entries=True, margins=(0.02, 0.02)):
         self.pathway = pathway
         self.show_maps = show_maps
         self.show_orthologs = show_orthologs
         self.show_compounds = show_compounds
         self.show_genes = show_genes
+        self.show_reaction_entries = show_reaction_entries
         self.label_compounds = label_compounds
         self.label_orthologs = label_orthologs
         self.label_reaction_entries = label_reaction_entries
@@ -122,7 +151,8 @@ class KGMLCanvas(object):
         # By default, they're slightly transparent.
         if self.show_maps:
             self.__add_maps()
-        self.__add_reaction_entries()
+        if self.show_reaction_entries:
+            self.__add_reaction_entries()
         if self.show_orthologs:
             self.__add_orthologs()
         if self.show_compounds:
@@ -176,7 +206,7 @@ class KGMLCanvas(object):
         # rectangle/roundrectangle, but Reportlab uses the co-ordinates of the
         # lower-left corner for rectangle/elif.
         if graphics.type == 'circle':
-            self.drawing.circle(graphics.x, graphics.y, graphics.width*0.5,
+            self.drawing.circle(graphics.x, graphics.y, graphics.width * 0.5,
                                 stroke=1, fill=1)
         elif graphics.type == 'roundrectangle':
             self.drawing.roundRect(graphics.x - graphics.width * 0.5,
@@ -233,30 +263,30 @@ class KGMLCanvas(object):
         """
         for ortholog in self.pathway.orthologs:
             for g in ortholog.graphics:
-                self.drawing.setStrokeColor(g.fgcolor)
-                self.drawing.setFillColor(g.bgcolor)
+                self.drawing.setStrokeColor(color_to_reportlab(g.fgcolor))
+                self.drawing.setFillColor(color_to_reportlab(g.bgcolor))
                 self.__add_graphics(g)
                 if self.label_orthologs:
                     # We want the label color to be slightly darker
                     # (where possible), so it can be read
-                    self.drawing.setFillColor(hexdarken(g.fgcolor))
+                    self.drawing.setFillColor(darken(g.fgcolor))
                     self.__add_labels(g)
 
     def __add_reaction_entries(self):
-        """Adds Entry elements corresponding to Reactions to the map drawing (PRIVATE).
+        """Adds Entry elements for Reactions to the map drawing (PRIVATE).
 
         In KGML, these are typically line objects, so we render them
         before the compound circles to cover the unsightly ends/junctions
         """
         for reaction in self.pathway.reaction_entries:
             for g in reaction.graphics:
-                self.drawing.setStrokeColor(g.fgcolor)
-                self.drawing.setFillColor(g.bgcolor)
+                self.drawing.setStrokeColor(color_to_reportlab(g.fgcolor))
+                self.drawing.setFillColor(color_to_reportlab(g.bgcolor))
                 self.__add_graphics(g)
                 if self.label_reaction_entries:
                     # We want the label color to be slightly darker
                     # (where possible), so it can be read
-                    self.drawing.setFillColor(hexdarken(g.fgcolor))
+                    self.drawing.setFillColor(darken(g.fgcolor))
                     self.__add_labels(g)
 
     def __add_compounds(self):
@@ -265,10 +295,10 @@ class KGMLCanvas(object):
             for g in compound.graphics:
                 # Modify transparency of compounds that don't participate
                 # in reactions
-                fillcolor = colors.HexColor(g.bgcolor)
+                fillcolor = color_to_reportlab(g.bgcolor)
                 if not compound.is_reactant:
                     fillcolor.alpha *= self.non_reactant_transparency
-                self.drawing.setStrokeColor(g.fgcolor)
+                self.drawing.setStrokeColor(color_to_reportlab(g.fgcolor))
                 self.drawing.setFillColor(fillcolor)
                 self.__add_graphics(g)
                 if self.label_compounds:
@@ -283,12 +313,11 @@ class KGMLCanvas(object):
         """Adds gene elements to the drawing of the map (PRIVATE)."""
         for gene in self.pathway.genes:
             for g in gene.graphics:
-                fillcolor = colors.HexColor(g.bgcolor)
-                self.drawing.setStrokeColor(g.fgcolor)
-                self.drawing.setFillColor(fillcolor)
+                self.drawing.setStrokeColor(color_to_reportlab(g.fgcolor))
+                self.drawing.setFillColor(color_to_reportlab(g.bgcolor))
                 self.__add_graphics(g)
                 if self.label_compounds:
-                    self.drawing.setFillColor(hexdarken(g.fgcolor))
+                    self.drawing.setFillColor(darken(g.fgcolor))
                     self.__add_labels(g)
 
     def __add_relations(self):
@@ -366,58 +395,3 @@ class KGMLCanvas(object):
         # print(bounds_from)
         # print(g_to)
         # print(bounds_to)
-
-
-if __name__ == '__main__':
-    # Test production of Reportlab Canvas PDF visualisation
-    # Try a default KO metabolic map with ortholog lines given
-    pathway = KGML_parser.read(open('ko01100.xml', 'rU'))
-    print(pathway)
-    kgml_map = KGMLCanvas(pathway)
-    kgml_map.draw('KGML_canvas_test.pdf')
-
-    # Try a Dickeya metabolic map with ortholog lines, modifying reaction
-    # graphics
-    pathway = KGML_parser.read(open('ddc01100.xml', 'rU'))
-    print(pathway)
-    kgml_map = KGMLCanvas(pathway)
-    # Set all reaction linewidths to 3 units
-    for r in pathway.reaction_entries:
-        for g in r.graphics:
-            g.width = 3
-    kgml_map.draw('KGML_canvas_ddc_test.pdf')
-
-    # Try a KO metabolic map with no ortholog lines, using a local .png
-    pathway = KGML_parser.read(open('ko_metabolic/ko00910.xml', 'rU'))
-    pathway.image = 'map/map00910.png'
-    print(pathway)
-    kgml_map = KGMLCanvas(pathway)
-    kgml_map.import_imagemap = True
-    kgml_map.show_maps = False
-    kgml_map.draw('KGML_canvas_map_local_test.pdf')
-
-    # Try a KO metabolic map with no ortholog lines, using the KEGG .png
-    pathway = KGML_parser.read(open('ko_metabolic/ko00253.xml', 'rU'))
-    print(pathway)
-    kgml_map = KGMLCanvas(pathway)
-    kgml_map.import_imagemap = True
-    kgml_map.show_maps = False
-    kgml_map.draw('KGML_canvas_map_test.pdf')
-
-    # Try a KO metabolic map with no ortholog lines using the KEGG .png,
-    # but this time using a Dickeya XML file
-    pathway = KGML_parser.read(open('dda00190.xml', 'rU'))
-    print(pathway)
-    kgml_map = KGMLCanvas(pathway)
-    kgml_map.import_imagemap = True
-    kgml_map.show_maps = True
-    kgml_map.draw('KGML_canvas_dda_map_test.pdf')
-
-    # Try a KO metabolic map with no ortholog lines using the KEGG .png,
-    # but this time using a Dickeya XML file
-    pathway = KGML_parser.read(open('test_retrieve_ddc00190.kgml', 'rU'))
-    print(pathway)
-    kgml_map = KGMLCanvas(pathway)
-    kgml_map.import_imagemap = True
-    kgml_map.show_maps = True
-    kgml_map.draw('KGML_canvas_ddc_map_test.pdf')
