@@ -2,9 +2,8 @@
 # This code is part of the Biopython distribution and governed by its
 # license.  Please see the LICENSE file that should have been included
 # as part of this package.
-"""
-This module provides code to work with the sprotXX.dat file from
-SwissProt.
+"""Code to work with the sprotXX.dat file from SwissProt.
+
 http://www.expasy.ch/sprot/sprot-top.html
 
 Tested with:
@@ -12,100 +11,111 @@ Release 56.9, 03-March-2009.
 
 
 Classes:
-Record             Holds SwissProt data.
-Reference          Holds reference data from a SwissProt record.
+
+    - Record             Holds SwissProt data.
+    - Reference          Holds reference data from a SwissProt record.
 
 Functions:
-read               Read one SwissProt record
-parse              Read multiple SwissProt records
+
+    - read               Read one SwissProt record
+    - parse              Read multiple SwissProt records
 
 """
 
+from __future__ import print_function
 
-class Record:
+from Bio._py3k import _as_string
+
+__docformat__ = "restructuredtext en"
+
+
+class Record(object):
     """Holds information from a SwissProt record.
 
     Members:
-    entry_name        Name of this entry, e.g. RL1_ECOLI.
-    data_class        Either 'STANDARD' or 'PRELIMINARY'.
-    molecule_type     Type of molecule, 'PRT',
-    sequence_length   Number of residues.
 
-    accessions        List of the accession numbers, e.g. ['P00321']
-    created           A tuple of (date, release).
-    sequence_update   A tuple of (date, release).
-    annotation_update A tuple of (date, release).
+        - entry_name        Name of this entry, e.g. RL1_ECOLI.
+        - data_class        Either 'STANDARD' or 'PRELIMINARY'.
+        - molecule_type     Type of molecule, 'PRT',
+        - sequence_length   Number of residues.
 
-    description       Free-format description.
-    gene_name         Gene name.  See userman.txt for description.
-    organism          The source of the sequence.
-    organelle         The origin of the sequence.
-    organism_classification  The taxonomy classification.  List of strings.
-                             (http://www.ncbi.nlm.nih.gov/Taxonomy/)
-    taxonomy_id       A list of NCBI taxonomy id's.
-    host_organism     A list of NCBI taxonomy id's of the hosts of a virus,
-                      if any.
-    references        List of Reference objects.
-    comments          List of strings.
-    cross_references  List of tuples (db, id1[, id2][, id3]).  See the docs.
-    keywords          List of the keywords.
-    features          List of tuples (key name, from, to, description).
-                      from and to can be either integers for the residue
-                      numbers, '<', '>', or '?'
+        - accessions        List of the accession numbers, e.g. ['P00321']
+        - created           A tuple of (date, release).
+        - sequence_update   A tuple of (date, release).
+        - annotation_update A tuple of (date, release).
+
+        - description       Free-format description.
+        - gene_name         Gene name.  See userman.txt for description.
+        - organism          The source of the sequence.
+        - organelle         The origin of the sequence.
+        - organism_classification  The taxonomy classification.  List of strings.
+          (http://www.ncbi.nlm.nih.gov/Taxonomy/)
+        - taxonomy_id       A list of NCBI taxonomy id's.
+        - host_organism     A list of names of the hosts of a virus, if any.
+        - host_taxonomy_id  A list of NCBI taxonomy id's of the hosts, if any.
+        - references        List of Reference objects.
+        - comments          List of strings.
+        - cross_references  List of tuples (db, id1[, id2][, id3]).  See the docs.
+        - keywords          List of the keywords.
+        - features          List of tuples (key name, from, to, description).
+          from and to can be either integers for the residue
+          numbers, '<', '>', or '?'
+
+        - seqinfo           tuple of (length, molecular weight, CRC32 value)
+        - sequence          The sequence.
 
-    seqinfo           tuple of (length, molecular weight, CRC32 value)
-    sequence          The sequence.
-    
     """
     def __init__(self):
         self.entry_name = None
         self.data_class = None
         self.molecule_type = None
         self.sequence_length = None
-        
+
         self.accessions = []
         self.created = None
         self.sequence_update = None
         self.annotation_update = None
-        
-        self.description = ''
+
+        self.description = []
         self.gene_name = ''
-        self.organism = ''
+        self.organism = []
         self.organelle = ''
         self.organism_classification = []
         self.taxonomy_id = []
         self.host_organism = []
+        self.host_taxonomy_id = []
         self.references = []
         self.comments = []
         self.cross_references = []
         self.keywords = []
         self.features = []
-        
+
         self.seqinfo = None
         self.sequence = ''
 
 
-class Reference:
+class Reference(object):
     """Holds information from one reference in a SwissProt entry.
 
     Members:
     number      Number of reference in an entry.
-    positions   Describes extent of work.  list of strings.
+    evidence    Evidence code.  List of strings.
+    positions   Describes extent of work.  List of strings.
     comments    Comments.  List of (token, text).
-    references  References.  List of (dbname, identifier)
+    references  References.  List of (dbname, identifier).
     authors     The authors of the work.
     title       Title of the work.
     location    A citation for the work.
-    
+
     """
     def __init__(self):
         self.number = None
         self.positions = []
         self.comments = []
         self.references = []
-        self.authors = ''
-        self.title = ''
-        self.location = ''
+        self.authors = []
+        self.title = []
+        self.location = []
 
 
 def parse(handle):
@@ -126,146 +136,154 @@ def read(handle):
         raise ValueError("More than one SwissProt record found")
     return record
 
- 
+
 # Everything below is considered private
 
 
 def _read(handle):
     record = None
+    unread = ""
     for line in handle:
+        # This is for Python 3 to cope with a binary handle (byte strings),
+        # or a text handle (unicode strings):
+        line = _as_string(line)
         key, value = line[:2], line[5:].rstrip()
-        if key=='**':
-            #See Bug 2353, some files from the EBI have extra lines
-            #starting "**" (two asterisks/stars).  They appear
-            #to be unofficial automated annotations. e.g.
-            #**
-            #**   #################    INTERNAL SECTION    ##################
-            #**HA SAM; Annotated by PicoHamap 1.88; MF_01138.1; 09-NOV-2003.
+        if unread:
+            value = unread + " " + value
+            unread = ""
+        if key == '**':
+            # See Bug 2353, some files from the EBI have extra lines
+            # starting "**" (two asterisks/stars).  They appear
+            # to be unofficial automated annotations. e.g.
+            # **
+            # **   #################    INTERNAL SECTION    ##################
+            # **HA SAM; Annotated by PicoHamap 1.88; MF_01138.1; 09-NOV-2003.
             pass
-        elif key=='ID':
+        elif key == 'ID':
             record = Record()
             _read_id(record, line)
             _sequence_lines = []
-        elif key=='AC':
+        elif key == 'AC':
             accessions = [word for word in value.rstrip(";").split("; ")]
             record.accessions.extend(accessions)
-        elif key=='DT':
+        elif key == 'DT':
             _read_dt(record, line)
-        elif key=='DE':
-            record.description += line[5:]
-        elif key=='GN':
-            record.gene_name += line[5:]
-        elif key=='OS':
-            record.organism += line[5:]
-        elif key=='OG':
+        elif key == 'DE':
+            record.description.append(value.strip())
+        elif key == 'GN':
+            if record.gene_name:
+                record.gene_name += " "
+            record.gene_name += value
+        elif key == 'OS':
+            record.organism.append(value)
+        elif key == 'OG':
             record.organelle += line[5:]
-        elif key=='OC':
+        elif key == 'OC':
             cols = [col for col in value.rstrip(";.").split("; ")]
             record.organism_classification.extend(cols)
-        elif key=='OX':
+        elif key == 'OX':
             _read_ox(record, line)
-        elif key=='OH':
+        elif key == 'OH':
             _read_oh(record, line)
-        elif key=='RN':
+        elif key == 'RN':
             reference = Reference()
             _read_rn(reference, value)
             record.references.append(reference)
-        elif key=='RP':
+        elif key == 'RP':
             assert record.references, "RP: missing RN"
             record.references[-1].positions.append(value)
-        elif key=='RC':
+        elif key == 'RC':
             assert record.references, "RC: missing RN"
             reference = record.references[-1]
-            _read_rc(reference, value)
-        elif key=='RX':
+            unread = _read_rc(reference, value)
+        elif key == 'RX':
             assert record.references, "RX: missing RN"
             reference = record.references[-1]
             _read_rx(reference, value)
-        elif key=='RL':
+        elif key == 'RL':
             assert record.references, "RL: missing RN"
             reference = record.references[-1]
-            reference.location += line[5:]
+            reference.location.append(value)
         # In UniProt release 1.12 of 6/21/04, there is a new RG
         # (Reference Group) line, which references a group instead of
         # an author.  Each block must have at least 1 RA or RG line.
-        elif key=='RA':
+        elif key == 'RA':
             assert record.references, "RA: missing RN"
             reference = record.references[-1]
-            reference.authors += line[5:]
-        elif key=='RG':
+            reference.authors.append(value)
+        elif key == 'RG':
             assert record.references, "RG: missing RN"
             reference = record.references[-1]
-            reference.authors += line[5:]
-        elif key=="RT":
+            reference.authors.append(value)
+        elif key == "RT":
             assert record.references, "RT: missing RN"
             reference = record.references[-1]
-            reference.title += line[5:]
-        elif key=='CC':
+            reference.title.append(value)
+        elif key == 'CC':
             _read_cc(record, line)
-        elif key=='DR':
+        elif key == 'DR':
             _read_dr(record, value)
-        elif key=='PE':
-            #TODO - Record this information?
+        elif key == 'PE':
+            # TODO - Record this information?
             pass
-        elif key=='KW':
-            cols = value.rstrip(";.").split('; ')
-            record.keywords.extend(cols)
-        elif key=='FT':
+        elif key == 'KW':
+            _read_kw(record, value)
+        elif key == 'FT':
             _read_ft(record, line)
-        elif key=='SQ':
+        elif key == 'SQ':
             cols = value.split()
             assert len(cols) == 7, "I don't understand SQ line %s" % line
             # Do more checking here?
             record.seqinfo = int(cols[1]), int(cols[3]), cols[5]
-        elif key=='  ':
+        elif key == '  ':
             _sequence_lines.append(value.replace(" ", "").rstrip())
-        elif key=='//':
-            # Remove trailing newlines
-            record.description = record.description.rstrip()
-            record.gene_name   = record.gene_name.rstrip()
-            record.organism    = record.organism.rstrip()
-            record.organelle   = record.organelle.rstrip()
+        elif key == '//':
+            # Join multiline data into one string
+            record.description = " ".join(record.description)
+            record.organism = " ".join(record.organism)
+            record.organelle = record.organelle.rstrip()
             for reference in record.references:
-                # Remove trailing newlines
-                reference.authors = reference.authors.rstrip()
-                reference.title = reference.title.rstrip()
-                reference.location = reference.location.rstrip()
+                reference.authors = " ".join(reference.authors).rstrip(";")
+                reference.title = " ".join(reference.title).rstrip(";")
+                if reference.title.startswith('"') and reference.title.endswith('"'):
+                    reference.title = reference.title[1:-1]  # remove quotes
+                reference.location = " ".join(reference.location)
             record.sequence = "".join(_sequence_lines)
             return record
         else:
-            raise ValueError("Unknown keyword %s found" % keyword)
+            raise ValueError("Unknown keyword '%s' found" % key)
     if record:
         raise ValueError("Unexpected end of stream.")
 
 
 def _read_id(record, line):
     cols = line[5:].split()
-    #Prior to release 51, included with MoleculeType:
-    #ID   EntryName DataClass; MoleculeType; SequenceLength AA.
+    # Prior to release 51, included with MoleculeType:
+    # ID   EntryName DataClass; MoleculeType; SequenceLength AA.
     #
-    #Newer files lack the MoleculeType:
-    #ID   EntryName DataClass; SequenceLength AA.
-    if len(cols) == 5 :
+    # Newer files lack the MoleculeType:
+    # ID   EntryName DataClass; SequenceLength AA.
+    if len(cols) == 5:
         record.entry_name = cols[0]
         record.data_class = cols[1].rstrip(";")
         record.molecule_type = cols[2].rstrip(";")
         record.sequence_length = int(cols[3])
-    elif len(cols) == 4 :
+    elif len(cols) == 4:
         record.entry_name = cols[0]
         record.data_class = cols[1].rstrip(";")
         record.molecule_type = None
         record.sequence_length = int(cols[2])
-    else :
-        raise ValueError("ID line has unrecognised format:\n"+line)
+    else:
+        raise ValueError("ID line has unrecognised format:\n" + line)
     # check if the data class is one of the allowed values
     allowed = ('STANDARD', 'PRELIMINARY', 'IPI', 'Reviewed', 'Unreviewed')
     if record.data_class not in allowed:
-        raise ValueError("Unrecognized data class %s in line\n%s" % \
+        raise ValueError("Unrecognized data class %s in line\n%s" %
               (record.data_class, line))
     # molecule_type should be 'PRT' for PRoTein
     # Note that has been removed in recent releases (set to None)
     if record.molecule_type not in (None, 'PRT'):
-        raise ValueError("Unrecognized molecule type %s in line\n%s" % \
+        raise ValueError("Unrecognized molecule type %s in line\n%s" %
               (record.molecule_type, line))
 
 
@@ -294,7 +312,7 @@ def _read_dt(record, line):
         uprcols = uprline.split()
         rel_index = -1
         for index in range(len(uprcols)):
-            if uprcols[index].find("REL.") >= 0:
+            if 'REL.' in uprcols[index]:
                 rel_index = index
         assert rel_index >= 0, \
                 "Could not find Rel. in DT line: %s" % line
@@ -305,18 +323,19 @@ def _read_dt(record, line):
         if str_version == '':
             version = 0
         # dot versioned
-        elif str_version.find(".") >= 0:
+        elif '.' in str_version:
             version = str_version
         # integer versioned
         else:
             version = int(str_version)
+        date = cols[0]
 
         if 'CREATED' in uprline:
-            record.created = cols[1], version
+            record.created = date, version
         elif 'LAST SEQUENCE UPDATE' in uprline:
-            record.sequence_update = cols[1], version
+            record.sequence_update = date, version
         elif 'LAST ANNOTATION UPDATE' in uprline:
-            record.annotation_update = cols[1], version
+            record.annotation_update = date, version
         else:
             assert False, "Shouldn't reach this line!"
     elif 'INTEGRATED INTO' in uprline \
@@ -337,24 +356,25 @@ def _read_dt(record, line):
         # DT   15-OCT-2001, sequence version 3.
         # DT   01-APR-2004, entry version 14.
         #
-        #This is a new style DT line...
+        # This is a new style DT line...
 
         # The date should be in string cols[1]
         # Get the version number if there is one.
         # For the three DT lines above: 0, 3, 14
         try:
             version = int(cols[-1])
-        except ValueError :
+        except ValueError:
             version = 0
+        date = cols[0].rstrip(",")
 
         # Re-use the historical property names, even though
         # the meaning has changed slighty:
-        if "INTEGRATED"  in uprline:
-            record.created = cols[1], version
+        if "INTEGRATED" in uprline:
+            record.created = date, version
         elif 'SEQUENCE VERSION' in uprline:
-            record.sequence_update = cols[1], version
+            record.sequence_update = date, version
         elif 'ENTRY VERSION' in uprline:
-            record.annotation_update = cols[1], version
+            record.annotation_update = date, version
         else:
             assert False, "Shouldn't reach this line!"
     else:
@@ -362,7 +382,7 @@ def _read_dt(record, line):
 
 
 def _read_ox(record, line):
-    # The OX line is in the format:
+    # The OX line used to be in the simple format:
     # OX   DESCRIPTION=ID[, ID]...;
     # If there are too many id's to fit onto a line, then the ID's
     # continue directly onto the next line, e.g.
@@ -372,6 +392,11 @@ def _read_ox(record, line):
     # To parse this, I need to check to see whether I'm at the
     # first line.  If I am, grab the description and make sure
     # it's an NCBI ID.  Then, grab all the id's.
+    #
+    # As of the 2014-10-01 release, there may be an evidence code, e.g.
+    # OX   NCBI_TaxID=418404 {ECO:0000313|EMBL:AEX14553.1};
+    # In the short term, we will ignore any evidence codes:
+    line = line.split('{')[0]
     if record.taxonomy_id:
         ids = line[5:].rstrip().rstrip(";")
     else:
@@ -382,35 +407,48 @@ def _read_ox(record, line):
 
 def _read_oh(record, line):
     # Line type OH (Organism Host) for viral hosts
-    # same code as in taxonomy_id()
-    if record.host_organism:
-        ids = line[5:].rstrip().rstrip(";")
-    else:
-        descr, ids = line[5:].rstrip().rstrip(";").split("=")
-        assert descr == "NCBI_TaxID", "Unexpected taxonomy type %s" % descr
-    record.host_organism.extend(ids.split(', '))
+    assert line[5:].startswith("NCBI_TaxID="), "Unexpected %s" % line
+    line = line[16:].rstrip()
+    assert line[-1] == "." and line.count(";") == 1, line
+    taxid, name = line[:-1].split(";")
+    record.host_taxonomy_id.append(taxid.strip())
+    record.host_organism.append(name.strip())
 
 
 def _read_rn(reference, rn):
-    assert rn[0] == '[' and rn[-1] == ']', "Missing brackets %s" % rn
-    reference.number = int(rn[1:-1])
-
+    # This used to be a very simple line with a reference number, e.g.
+    # RN   [1]
+    # As of the 2014-10-01 release, there may be an evidence code, e.g.
+    # RN   [1] {ECO:0000313|EMBL:AEX14553.1}
+    words = rn.split(None, 1)
+    number = words[0]
+    assert number.startswith('[') and number.endswith(']'), "Missing brackets %s" % number
+    reference.number = int(number[1:-1])
+    if len(words) > 1:
+        evidence = words[1]
+        assert evidence.startswith('{') and evidence.endswith('}'), "Missing braces %s" % evidence
+        reference.evidence = evidence[1:-1].split('|')
 
 def _read_rc(reference, value):
     cols = value.split(';')
+    if value[-1] == ';':
+        unread = ""
+    else:
+        cols, unread = cols[:-1], cols[-1]
     for col in cols:
         if not col:  # last column will be the empty string
             return
         # The token is everything before the first '=' character.
         i = col.find("=")
-        if i>=0:
-            token, text = col[:i], col[i+1:]
+        if i >= 0:
+            token, text = col[:i], col[i + 1:]
             comment = token.lstrip(), text
             reference.comments.append(comment)
         else:
             comment = reference.comments[-1]
             comment = "%s %s" % (comment, col)
             reference.comments[-1] = comment
+    return unread
 
 
 def _read_rx(reference, value):
@@ -422,7 +460,7 @@ def _read_rx(reference, value):
     # have extraneous information in the RX line.  Check for
     # this and chop it out of the line.
     # (noticed by katel@worldpath.net)
-    value = value.replace(' [NCBI, ExPASy, Israel, Japan]','')
+    value = value.replace(' [NCBI, ExPASy, Israel, Japan]', '')
 
     # RX lines can also be used of the form
     # RX   PubMed=9603189;
@@ -431,53 +469,68 @@ def _read_rx(reference, value):
     # RX   MEDLINE=95385798; PubMed=7656980;
     # RX   PubMed=15060122; DOI=10.1136/jmg 2003.012781;
     # We look for these cases first and deal with them
+    warn = False
     if "=" in value:
         cols = value.split("; ")
         cols = [x.strip() for x in cols]
         cols = [x for x in cols if x]
         for col in cols:
             x = col.split("=")
-            assert len(x) == 2, "I don't understand RX line %s" % line
-            key, value = x[0], x[1].rstrip(";")
-            reference.references.append((key, value))
+            if len(x) != 2 or x == ("DOI", "DOI"):
+                warn = True
+                break
+            assert len(x) == 2, "I don't understand RX line %s" % value
+            reference.references.append((x[0], x[1].rstrip(";")))
     # otherwise we assume we have the type 'RX   MEDLINE; 85132727.'
     else:
         cols = value.split("; ")
         # normally we split into the three parts
-        assert len(cols) == 2, "I don't understand RX line %s" % line
-        reference.references.append((cols[0].rstrip(";"), cols[1].rstrip(".")))
+        if len(cols) != 2:
+            warn = True
+        else:
+            reference.references.append((cols[0].rstrip(";"), cols[1].rstrip(".")))
+    if warn:
+        import warnings
+        from Bio import BiopythonParserWarning
+        warnings.warn("Possibly corrupt RX line %r" % value,
+                      BiopythonParserWarning)
 
 
 def _read_cc(record, line):
-    key, value = line[5:8], line[8:]
-    if key=='-!-':   # Make a new comment
+    key, value = line[5:8], line[9:].rstrip()
+    if key == '-!-':   # Make a new comment
         record.comments.append(value)
-    elif key=='   ': # add to the previous comment
+    elif key == '   ':  # add to the previous comment
         if not record.comments:
             # TCMO_STRGA in Release 37 has comment with no topic
             record.comments.append(value)
         else:
-            record.comments[-1] += value
-    elif key=='---':
-        # If there are no comments, and it's not the closing line,
-        # make a new comment.
-        if not record.comments or record.comments[-1][:3] != '---':
-            record.comments.append(line[5:])
-        else:
-            record.comments[-1] += line[5:]
-    else:  # copyright notice
-        record.comments[-1] += line[5:]
+            record.comments[-1] += " " + value
 
 
 def _read_dr(record, value):
-    # Remove the comments at the end of the line
-    i = value.find(' [')
-    if i >= 0:
-        value = value[:i]
     cols = value.rstrip(".").split('; ')
     record.cross_references.append(tuple(cols))
 
 
+def _read_kw(record, value):
+    # Old style - semi-colon separated, multi-line. e.g. Q13639.txt
+    # KW   Alternative splicing; Cell membrane; Complete proteome;
+    # KW   Disulfide bond; Endosome; G-protein coupled receptor; Glycoprotein;
+    # KW   Lipoprotein; Membrane; Palmitate; Polymorphism; Receptor; Transducer;
+    # KW   Transmembrane.
+    #
+    # New style as of 2014-10-01 release with evidence codes, e.g. H2CNN8.txt
+    # KW   Monooxygenase {ECO:0000313|EMBL:AEX14553.1};
+    # KW   Oxidoreductase {ECO:0000313|EMBL:AEX14553.1}.
+    # For now to match the XML parser, drop the evidence codes.
+    for value in value.rstrip(";.").split('; '):
+        if value.endswith("}"):
+            # Discard the evidence code
+            value = value.rsplit("{", 1)[0]
+        record.keywords.append(value.strip())
+
+
 def _read_ft(record, line):
     line = line[5:]    # get rid of junk in front
     name = line[0:8].rstrip()
@@ -489,17 +542,18 @@ def _read_ft(record, line):
         to_res = int(line[16:22])
     except ValueError:
         to_res = line[16:22].lstrip()
-    description = line[29:70].rstrip()
-    #if there is a feature_id (FTId), store it away
-    if line[29:35]==r"/FTId=":
+    # if there is a feature_id (FTId), store it away
+    if line[29:35] == r"/FTId=":
         ft_id = line[35:70].rstrip()[:-1]
+        description = ""
     else:
-        ft_id =""
+        ft_id = ""
+        description = line[29:70].rstrip()
     if not name:  # is continuation of last one
         assert not from_res and not to_res
-        name, from_res, to_res, old_description,old_ft_id = record.features[-1]
+        name, from_res, to_res, old_description, old_ft_id = record.features[-1]
         del record.features[-1]
-        description = "%s %s" % (old_description, description)
+        description = ("%s %s" % (old_description, description)).strip()
 
         # special case -- VARSPLIC, reported by edvard@farmasi.uit.no
         if name == "VARSPLIC":
@@ -523,26 +577,25 @@ def _read_ft(record, line):
                 second_seq = second_seq.replace(" ", "")
                 # reassemble the description
                 description = first_seq + " -> " + second_seq + extra_info
-    record.features.append((name, from_res, to_res, description,ft_id))
+    record.features.append((name, from_res, to_res, description, ft_id))
 
 
-if __name__ == "__main__" :
-    print "Quick self test..."
+if __name__ == "__main__":
+    print("Quick self test...")
 
     example_filename = "../../Tests/SwissProt/sp008"
 
     import os
     if not os.path.isfile(example_filename):
-        print "Missing test file %s" % example_filename
-    else :
-        #Try parsing it!
-        
-        handle = open(example_filename)
-        records = parse(handle)
-        for record in records:
-            print record.entry_name
-            print ",".join(record.accessions)
-            print record.keywords
-            print repr(record.organism)
-            print record.sequence[:20] + "..."
-        handle.close()
+        print("Missing test file %s" % example_filename)
+    else:
+        # Try parsing it!
+
+        with open(example_filename) as handle:
+            records = parse(handle)
+            for record in records:
+                print(record.entry_name)
+                print(",".join(record.accessions))
+                print(record.keywords)
+                print(repr(record.organism))
+                print(record.sequence[:20] + "...")
